@@ -167,6 +167,8 @@ sub computePlatformDefault($) {
     } elsif ($OSNAME eq "aix" || $OSNAME eq "freebsd" || $OSNAME eq "darwin" ||
              $OSNAME eq "amigaos") {
         $$defaultP = $OSNAME;
+    } elsif ($OSNAME eq "solaris") {
+        $$defaultP = "sun";
     } elsif ($OSNAME eq "dec_osf") {
         $$defaultP = "tru64";
     } else {
@@ -265,7 +267,8 @@ sub getPlatform() {
 
 
 
-sub getCompiler($) {
+sub getCompiler($$) {
+    my ($platform, $compilerR) = @_;
 #-----------------------------------------------------------------------------
 #  Here are some of the issues surrounding choosing a compiler:
 #
@@ -294,14 +297,11 @@ sub getCompiler($) {
 # compiler.  It is simply "cc" to mean native compiler or "gcc" to mean
 # GNU compiler or undefined to express no preference.
 #-----------------------------------------------------------------------------
-    my ($platform) = @_;
-    
     my %gccCapablePlatform = ("SOLARIS" => 1,
                               "TRU64"   => 1,
                               "SCO"     => 1,
-                              "AIX"     => 1);
-
-    my $compiler;
+                              "AIX"     => 1,
+                              "HP"      => 1);
 
     if ($gccCapablePlatform{$platform}) {
         print("GNU compiler or native operating system compiler (cc)?\n");
@@ -318,16 +318,16 @@ sub getCompiler($) {
         my $response = prompt("gcc or cc", $default);
 
         if ($response eq "gcc") {
-            $compiler = "gcc";
+            $$compilerR = "gcc";
         } elsif ($response eq "cc") {
-            $compiler = "cc";
+            $$compilerR = "cc";
         } else {
             print("'$response' isn't one of the choices.  \n" .
                   "You must choose 'gcc' or 'cc'.\n");
             exit 12;
         }
 
-        if ($compiler eq "gcc" && !commandExists("gcc")) {
+        if ($$compilerR eq "gcc" && !commandExists("gcc")) {
             print("WARNING: You selected the GNU compiler, " .
                   "but do not appear to have a program " .
                   "named 'gcc' in your PATH.  This may " .
@@ -337,22 +337,81 @@ sub getCompiler($) {
         }
         print("\n");
     }
-
-    return($compiler);
 }
 
 
-sub getLinker($) {
 
-    my ($platform) = @_;
+sub gccLinker() {
+#-----------------------------------------------------------------------------
+#  Determine what linker Gcc on this system appears to use.
+#
+#  Return either "gnu" or "sun"
+#
+#  For now, we assume it must be either a GNU linker or Sun linker and
+#  that all Sun linkers are fungible.
+#
+#  If we can't tell, we assume it is the GNU linker.
+#-----------------------------------------------------------------------------
+    # First, we assume that the compiler calls 'collect2' as the linker
+    # front end.  The specs file might specify some other program, but
+    # it usually says 'collect2'.
+    
+    my $retval;
 
-    my $linker;
+    my $collect2 = qx{gcc --print-prog-name=collect2};
+    
+    if (defined($collect2)) {
+        chomp($collect2);
+        my $linker = qx{$collect2 -v 2>&1};
+        if (defined($linker) && $linker =~ m{GNU ld}) {
+            $retval = "gnu";
+        } else {
+            $retval = "sun";
+        }
+    } else {
+        $retval = "gnu";
+    }
+    return $retval;
+}
+
+
+
+sub getLinker($$$$) {
+
+    my ($platform, $compiler, $baseLinkerR, $viaCompilerR) = @_;
+
+    my $baseLinker;
 
     if ($platform eq "SOLARIS") {
-        $linker = "SUNVIACOMPILER";
-    }
+        $$viaCompilerR = $TRUE;
 
-    return($linker);
+        while (!defined($$baseLinkerR)) {
+            print("GNU linker or SUN linker?\n");
+            print("\n");
+
+            my $default;
+            
+            if ($compiler eq "gcc") {
+                $default = gccLinker();
+            } else {
+                $default = "sun";
+            }
+            my $response = prompt("sun or gnu", $default);
+            
+            if ($response eq "gnu") {
+                $$baseLinkerR = "GNU";
+            } elsif ($response eq "sun") {
+                $$baseLinkerR = "SUN";
+            } else {
+                print("'$response' isn't one of the choices.  \n" .
+                      "You must choose 'sun' or 'gnu'.\n");
+            }
+            print("\n");
+        }
+    } else {
+        $$viaCompilerR = $TRUE;
+        $$baseLinkerR = "?";
+    }
 }
 
 
@@ -861,16 +920,41 @@ sub chooseTestCompiler($) {
 
 
 
-sub testCompileJpeglibH($$) {
-    my ($cc, $cflags) = @_;
+sub testCflags($) {
+    my ($needLocal) = @_;
+
+    my $cflags;
+
+    $cflags = "";  # initial value 
+    
+    if ($ENV{"CPPFLAGS"}) {
+        $cflags = $ENV{"CPPFLAGS"};
+    } else {
+        $cflags = "";
+    }
+    
+    if ($ENV{"CFLAGS"}) {
+        $cflags .= " " . $ENV{"CFLAGS"};
+    }
+    
+    if ($needLocal) {
+        $cflags .= " -I/usr/local/include";
+    }
+    return $cflags;
+}    
+
+
+
+sub testCompile($$$$) {
+    my ($cc, $cflags, $cSourceCodeR, $successR) = @_;
 #-----------------------------------------------------------------------------
-#  Do a test compile to see if we can see jpeglib.h.
+#  Do a test compile of the program in @{$cSourceCodeR}.
+#  
+#  Return $$successR == $TRUE iff the compile succeeds (exit code 0).
 #-----------------------------------------------------------------------------
     my ($cFile, $cFileName) = tempFile(".c");
 
-    print $cFile ("#include <ctype.h>\n");
-    print $cFile ("#include <stdio.h>\n");
-    print $cFile ("#include <jpeglib.h>\n");
+    print $cFile @{$cSourceCodeR};
     
     my ($oFile, $oFileName) = tempFile(".o");
     # Note: we tried using /dev/null for the output file and got complaints
@@ -885,53 +969,56 @@ sub testCompileJpeglibH($$) {
     unlink($cFileName);
     close($cFile);
 
-    return ($rc == 0);
+    $$successR = ($rc == 0);
 }
 
 
 
-sub testCompileJpegMarkerStruct($$) {
-    my ($cc, $cflags) = @_;
+sub testCompileJpeglibH($$$) {
+    my ($cc, $cflags, $successR) = @_;
+#-----------------------------------------------------------------------------
+#  Do a test compile to see if we can see jpeglib.h.
+#-----------------------------------------------------------------------------
+    my @cSourceCode = (
+                       "#include <ctype.h>\n",
+                       "#include <stdio.h>\n",
+                       "#include <jpeglib.h>\n",
+                       );
+    
+    testCompile($cc, $cflags, \@cSourceCode, $successR);
+}
+
+
+
+sub testCompileJpegMarkerStruct($$$) {
+    my ($cc, $cflags, $successR) = @_;
 #-----------------------------------------------------------------------------
 #  Do a test compile to see if struct jpeg_marker_struct is defined in 
 #  jpeglib.h.  Assume it is already established that the compiler works
 #  and can find jpeglib.h.
 #-----------------------------------------------------------------------------
-    my ($cFile, $cFileName) = tempFile(".c");
+    my @cSourceCode = (
+                       "#include <ctype.h>\n",
+                       "#include <stdio.h>\n",
+                       "#include <jpeglib.h>\n",
+                       "struct jpeg_marker_struct test;\n",
+                       );
 
-    print $cFile ("#include <ctype.h>\n");
-    print $cFile ("#include <stdio.h>\n");
-    print $cFile ("#include <jpeglib.h>\n");
-    print $cFile ("struct jpeg_marker_struct test;\n");
-    
-    my ($oFile, $oFileName) = tempFile(".o");
-    # Note: we tried using /dev/null for the output file and got complaints
-    # from the Sun compiler that it has the wrong suffix.  2002.08.09.
-    
-    my $compileCommand = "$cc -c -o $oFileName $cflags $cFileName";
-    print ("Doing test compile: $compileCommand\n");
-    my $rc = system($compileCommand);
-    
-    unlink($oFileName);
-    close($oFile);
-    unlink($cFileName);
-    close($cFile);
-
-    return ($rc == 0);
+    testCompile($cc, $cflags, \@cSourceCode, $successR);
 }
 
 
 
-sub printJpegIncludeWarning($) {
+sub printMissingHdrWarning($$) {
 
-    my ($jpeg_hdr_dir) = @_;
+    my ($name, $hdr_dir) = @_;
 
-    print("WARNING: You said the compile-time part of the JPEG library " .
+    print("WARNING: You said the compile-time part of the $name library " .
           "(the header\n");
     print("files) is in ");
 
-    if (defined($jpeg_hdr_dir)) {
-        print("directory '$jpeg_hdr_dir', ");
+    if (defined($hdr_dir)) {
+        print("directory '$hdr_dir', ");
     } else {
         print("the compiler's default search path, ");
     }
@@ -940,8 +1027,8 @@ sub printJpegIncludeWarning($) {
           "compile might\n");
     print("just be wrong, but otherwise the Netpbm build will fail.\n");
     print("\n");
-    print("To fix this, either install the JPEG library there \n");
-    print("or re-run Configure and answer the question about the JPEG " .
+    print("To fix this, either install the $name library there \n");
+    print("or re-run Configure and answer the question about the $name " .
           "library\n");
     print("differently.\n");
     print("\n");
@@ -967,41 +1054,26 @@ sub printOldJpegWarning() {
 
 sub testJpegHdr($$$) {
 
-    my ($compiler, $needLocal, $jpeg_hdr_dir) = @_;
+    my ($compiler, $needLocal, $jpeghdr_dir) = @_;
 
     my $cc = chooseTestCompiler($compiler);
 
     if (defined($cc)) {
 
-        my $cflags;
+        my $generalCflags = testCflags($needLocal);
 
-        if ($ENV{"CPPFLAGS"}) {
-            $cflags = $ENV{"CPPFLAGS"};
-        } else {
-            $cflags = "";
-        }
+        my $jpegIOpt = $jpeghdr_dir ? "-I$jpeghdr_dir" : "";
 
-        if ($ENV{"CFLAGS"}) {
-            $cflags .= " " . $ENV{"CFLAGS"};
-        }
-
-        if (defined($jpeg_hdr_dir)) {
-            $cflags .= " -I$jpeg_hdr_dir";
-        }
-
-        if ($needLocal) {
-            $cflags .= " -I/usr/local/include";
-        }
-        
-        my $success = testCompileJpeglibH($cc, $cflags);
+        testCompileJpeglibH($cc, "$generalCflags $jpegIOpt", \my $success);
 
         if (!$success) {
             print("\n");
-            printJpegIncludeWarning($jpeg_hdr_dir);
+            printMissingHdrWarning("JPEG", $jpeghdr_dir);
         } else {
             # We can get to something named jpeglib.h, but maybe it's an old
             # version we can't use.  Check it out.
-            my $success = testCompileJpegMarkerStruct($cc, $cflags);
+            testCompileJpegMarkerStruct($cc, "$generalCflags $jpegIOpt", 
+                                        \my $success);
             if (!$success) {
                 print("\n");
                 printOldJpegWarning();
@@ -1012,20 +1084,87 @@ sub testJpegHdr($$$) {
 
 
 
-sub testConfiguration($$$$) {
+sub testCompileZlibH($$$) {
+    my ($cc, $cflags, $successR) = @_;
+#-----------------------------------------------------------------------------
+#  Do a test compile to see if we can see zlib.h.
+#-----------------------------------------------------------------------------
+    my @cSourceCode = (
+                       "#include <zlib.h>\n",
+                       );
+    
+    testCompile($cc, $cflags, \@cSourceCode, $successR);
+}
 
-    my ($compiler, $needLocal, $jpeglib, $jpeg_hdr_dir) = @_;
+
+
+sub testCompilePngH($$$) {
+    my ($cc, $cflags, $successR) = @_;
+#-----------------------------------------------------------------------------
+#  Do a test compile to see if we can see png.h, assuming we can see
+#  png.h, which png.h #includes.
+#-----------------------------------------------------------------------------
+    my @cSourceCode = (
+                       "#include <png.h>\n",
+                       );
+    
+    testCompile($cc, $cflags, \@cSourceCode, $successR);
+}
+
+
+
+sub testPngHdr($$$$) {
+#-----------------------------------------------------------------------------
+#  Issue a warning if the compiler can't find png.h.
+#-----------------------------------------------------------------------------
+    my ($compiler, $needLocal, $pnghdr_dir, $zhdr_dir) = @_;
+
+    my $cc = chooseTestCompiler($compiler);
+
+    if (defined($cc)) {
+
+        my $generalCflags = testCflags($needLocal);
+
+        my $zlibIOpt = $zhdr_dir ? "-I$zhdr_dir" : "";
+
+        testCompileZlibH($cc, "$generalCflags $zlibIOpt", \my $success);
+        if (!$success) {
+            print("\n");
+            printMissingHdrWarning("Zlib", $zhdr_dir);
+        } else {
+            my $pngIOpt = $pnghdr_dir ? "-I$pnghdr_dir" : "";
+
+            testCompilePngH($cc, 
+                            "$generalCflags $zlibIOpt $pngIOpt", 
+                            \my $success);
+
+            if (!$success) {
+                print("\n");
+                printMissingHdrWarning("PNG", $pnghdr_dir);
+            }
+        }
+    }
+}
+
+
+
+sub testConfiguration($$$$$$$$) {
+
+    my ($compiler, $needLocal, $jpeglib, $jpeghdr_dir,
+        $pnglib, $pnghdr_dir, $zlib, $zhdr_dir) = @_;
 
     if (defined($jpeglib)) {
-        testJpegHdr($compiler, $needLocal, $jpeg_hdr_dir);
+        testJpegHdr($compiler, $needLocal, $jpeghdr_dir);
+    }
+    if (defined($pnglib) && defined($zlib)) {
+        testPngHdr($compiler, $needLocal, $pnghdr_dir, $zhdr_dir);
     }
 
-    # TODO: We ought to validate other libraries too.  But this is
-    # still rather experimental.  Besides, in the vast majority of
-    # cases where the user incorrectly identifies any library, it
-    # affects all the libraries and if the user can get a handle on the
-    # JPEG library problem, he will also solve problems with any other
-    # library.
+    # TODO: We ought to validate other libraries too.  But it's not
+    # that important, because in the vast majority of cases where the
+    # user incorrectly identifies any library, it affects all the
+    # libraries and if the user can get a handle on the JPEG library
+    # problem, he will also solve problems with any other library.
 }
 
 
@@ -1061,6 +1200,7 @@ if (stat("Makefile.config")) {
     }
     chomp($answer);
     if (uc($answer) ne "Y") {
+        print("Aborting at user request.\n");
         exit(1);
     }
 }
@@ -1082,8 +1222,8 @@ if ($platform eq "NONE") {
     exit;
 }
 
-my $compiler = getCompiler($platform);
-my $linker = getLinker($platform);
+getCompiler($platform, \my $compiler);
+getLinker($platform, $compiler, \my $baseLinker, \my $linkViaCompiler);
 
 my $netpbmlib_runtime_path;
     # Undefined if the default from Makefile.config.in is acceptable.
@@ -1093,7 +1233,7 @@ if ($platform eq "SOLARIS" or $platform eq "IRIX" or
     $platform eq "AMIGA") {
     print("Where will the Netpbm shared library reside once installed?\n");
     print("Enter 'default' if it will reside somewhere that the shared\n");
-    print("library loader will find them automatically.  Otherwise, \n");
+    print("library loader will find it automatically.  Otherwise, \n");
     print("this directory will get built into the Netpbm programs.\n");
     print("\n");
 
@@ -1297,7 +1437,11 @@ validateLibraries($jpeglib, $tifflib, $pnglib, $zlib);
 
 warnJpegTiffDependency($jpeglib, $tifflib);
 
-testConfiguration($compiler, needLocal($platform), $jpeglib, $jpeghdr_dir);
+testConfiguration($compiler, needLocal($platform), 
+                  $jpeglib, $jpeghdr_dir,
+                  $pnglib, $pnghdr_dir,
+                  $zlib, $zhdr_dir,
+                  );
 
 #******************************************************************************
 #
@@ -1422,16 +1566,33 @@ if ($platform eq "GNU") {
     if ($compiler eq "cc") {
         push(@Makefile_config, "CFLAGS = -O\n");
         push(@Makefile_config, "CFLAGS_SHLIB = -Kpic\n");
-        push(@Makefile_config, "RPATHOPTNAME = -R\n");
     } else {
         push(@Makefile_config, "CC = gcc\n");
         push(@Makefile_config, gnuCflags());
+    }
+    # Before Netpbm 10.20 (January 2004), we set this to -R for 
+    # $compiler == cc and -rpath otherwise.  But now we know that the GNU
+    # compiler can also invoke a linker that needs -R, so we're more flexible.
+    if ($baseLinker eq "GNU") {
         push(@Makefile_config, "RPATHOPTNAME = -rpath\n");
+    } else {
+        push(@Makefile_config, "RPATHOPTNAME = -R\n");
     }
     push(@Makefile_config, "NETWORKLD = -lsocket -lnsl\n");
 } elsif ($platform eq "HP-UX") {
-    push(@Makefile_config, 'CFLAGS = -O3 -fPIC $(CADD)', "\n");
-    push(@Makefile_config, 'LDFLAGS = -Wl,+b,/usr/pubsw/lib', "\n");
+    if ($compiler eq "gcc") {
+        push(@Makefile_config, "CC = gcc\n");
+        push(@Makefile_config, gnuCflags());
+        push(@Makefile_config, "CFLAGS += -fPIC\n");
+        push(@Makefile_config, "LDSHLIB = -shared -fPIC\n");
+        push(@Makefile_config, 'LDFLAGS = -Wl,+b,/usr/pubsw/lib', "\n");
+    } else {
+        # We don't know what to do here.  We used to (before 10.20) just
+        # just assume the compiler was gcc.  We know that the gcc stuff
+        # above does NOT work for HP native compiler.
+        push(@Makefile_config, "CFLAGS = \n");
+        push(@Makefile_config, "LDFLAGS = \n");
+    }
 } elsif ($platform eq "AIX") {
     push(@Makefile_config, 'LDFLAGS = -L /usr/pubsw/lib', "\n");
     if ($compiler eq "cc") {
@@ -1542,7 +1703,7 @@ if (needLocal($platform)) {
     push(@Makefile_config, "LDFLAGS += -L/usr/local/lib\n");
 }
 
-if (defined($linker) && $linker eq "SUNVIACOMPILER") {
+if ($linkViaCompiler) {
     push(@Makefile_config, "LINKERISCOMPILER = Y\n");
 }
 
