@@ -56,21 +56,132 @@ static void read_fits_header ARGS(( FILE* fp, struct FITS_Header* hP ));
 static void read_card ARGS(( FILE* fp, char* buf ));
 static void read_val ARGS(( FILE* fp, int bitpix, double* vp ));
      
+
+
+static void
+scanImageForMinMax(FILE *       const ifP,
+                   unsigned int const images,
+                   int          const cols,
+                   int          const rows,
+                   unsigned int const bitpix,
+                   double       const bscale,
+                   double       const bzer,
+                   unsigned int const imagenum,
+                   bool         const multiplane,
+                   double *     const dataminP,
+                   double *     const datamaxP) {
+
+    double dmax, dmin;
+    unsigned int image;
+    pm_filepos rasterPos;
+    double fmaxval;
+    
+    pm_tell2(ifP, &rasterPos, sizeof(rasterPos));
+
+    pm_message("Scanning file for scaling parameters");
+
+    switch (bitpix) {
+    case   8: fmaxval = 255.0;        break;
+    case  16: fmaxval = 65535.0;      break;
+    case  32: fmaxval = 4294967295.0; break;
+    case -32: fmaxval = FLT_MAX;      break;
+    case -64: fmaxval = DBL_MAX;      break;
+    default:
+        pm_error("unusual bits per pixel (%u), can't read", bitpix);
+    }
+
+    dmax = -fmaxval;
+    dmin = fmaxval;
+    for (image = 1; image <= images; ++image) {
+        unsigned int row;
+        for (row = 0; row < rows; ++row) {
+            unsigned int col;
+            for (col = 0; col < cols; ++col) {
+                double val;
+                read_val(ifP, bitpix, &val);
+                if (image == imagenum || multiplane ) {
+                    dmax = MAX(dmax, val);
+                    dmin = MIN(dmin, val);
+                }
+            }
+        }
+        if (bscale < 0.0) {
+            double const origDmax = dmax;
+            dmax = dmin;
+            dmin = origDmax;
+        }
+    }
+    *dataminP = dmin * bscale + bzer;
+    *datamaxP = dmax * bscale + bzer;
+
+    pm_message("Scan results: min=%f max=%f", *dataminP, *datamaxP);
+
+    pm_seek2(ifP, &rasterPos, sizeof(rasterPos));
+}
+
+
+
+static void
+computeMinMax(FILE *             const ifP,
+              unsigned int       const images,
+              int                const cols,
+              int                const rows,
+              struct FITS_Header const h,
+              unsigned int       const imagenum,
+              bool               const multiplane,
+              bool               const forcemin,
+              bool               const forcemax,
+              double             const frmin,
+              double             const frmax,
+              double *           const dataminP,
+              double *           const datamaxP) {
+
+    double datamin, datamax;
+
+    datamin = -DBL_MAX;  /* initial assumption */
+    datamax = DBL_MAX;   /* initial assumption */
+
+    if (forcemin)
+        datamin = frmin;
+    if (forcemax)
+        datamax = frmax;
+
+    if (datamin == -DBL_MAX)
+        datamin = h.datamin;
+    if (datamax == DBL_MAX)
+        datamax = h.datamax;
+
+    if (datamin == -DBL_MAX || datamax == DBL_MAX) {
+        double scannedDatamin, scannedDatamax;
+        scanImageForMinMax(ifP, images, cols, rows, h.bitpix, h.bscale, h.bzer,
+                           imagenum, multiplane,
+                           &scannedDatamin, &scannedDatamax);
+
+        if (datamin == -DBL_MAX)
+            datamin = scannedDatamin;
+        if (datamax == DBL_MAX)
+            datamax = scannedDatamax;
+    }
+    *dataminP = datamin;
+    *datamaxP = datamax;
+}
+
+
+
 int
-main( argc, argv )
-    int argc;
-    char* argv[];
-{
+main(int argc, char * argv[]) {
+
     FILE* ifp;
     int argn, imagenum, image, row;
     register int col;
     xelval maxval;
-    double val, fmaxval, frmin, frmax, dmax, dmin, scale, t;
+    double val, frmin, frmax, scale, t;
+    double datamin, datamax;
     int rows, cols, images, format;
     struct FITS_Header h;
     xel** pnmarray;
     xelval tx, txv[4];
-    char* fits_name;
+    const char* fits_name;
     const char* const usage = "[-image N] [-scanmax] [-printmax] [-min f] [-max f] [FITSfile]";
 
     int doscan = 0;
@@ -78,6 +189,7 @@ main( argc, argv )
     int forcemin = 0;
     int forcemax = 0;
     int printmax = 0;
+    bool multiplane;
   
     pnm_init( &argc, argv );
   
@@ -123,45 +235,20 @@ main( argc, argv )
     if ( argn < argc )
     {
         fits_name = argv[argn];
-        ifp = pm_openr( fits_name );
         ++argn;
     }
     else
-        ifp = stdin;
+        fits_name = "-";
 
     if ( argn != argc )
         pm_usage( usage );
+
+    ifp = pm_openr_seekable(fits_name);
   
     read_fits_header( ifp, &h );
   
     if ( ! h.simple )
         pm_error( "FITS file is not in simple format, can't read" );
-    switch ( h.bitpix )
-    {
-    case 8:
-        fmaxval = 255.0;
-        break;
-      
-    case 16:
-        fmaxval = 65535.0;
-        break;
-      
-    case 32:
-        fmaxval = 4294967295.0;
-        break;
-      
-    case -32:
-        fmaxval = FLT_MAX;
-        break;
-      
-    case -64:
-        fmaxval = DBL_MAX;
-        break;
-      
-    default:
-        pm_error( "unusual bits per pixel (%d), can't read", h.bitpix );
-    }
-
     if ( h.naxis != 2 && h.naxis != 3 )
         pm_message( "Warning: FITS file has %d axes", h.naxis );
     cols = h.naxis1;
@@ -176,56 +263,27 @@ main( argc, argv )
     if ( images != 3 && images > 1 && imagenum == 0 )
         pm_error( "need to specify a plane using the -imagenum flag" );
 
-    format = PGM_FORMAT;
-    if ( images == 3 && imagenum == 0 ) 
-        format = PPM_FORMAT;
+    multiplane = ( images == 3 && imagenum == 0 );
 
-    if (forcemin) h.datamin = frmin;
-    if (forcemax) h.datamax = frmax;
+    computeMinMax(ifp, images, cols, rows, h, imagenum, multiplane,
+                  forcemin, forcemax, frmin, frmax,
+                  &datamin, &datamax);
 
-    if ( doscan || h.datamin == - DBL_MAX || h.datamax == DBL_MAX )
-    {
-        if ( ifp == stdin )
-            pm_message( "warning: cannot scan max and read from stdin" );
-        else 
-        {
-            /* OK, We have to scale this the hard way. */
-            pm_message( "Scanning file for scaling parameters" );
-            dmax = -fmaxval;
-            dmin = fmaxval;
-            for ( image = 1; image <= images; ++image )
-            {
-                for ( row = 0; row < rows; ++row)
-                {
-                    for ( col = 0; col < cols; ++col)
-                    {
-                        read_val (ifp, h.bitpix, &val);
-                        if ( image == imagenum || format == PPM_FORMAT ) {
-                            if ( val > dmax ) dmax = val;
-                            if ( val < dmin ) dmin = val;
-                        }
-                    }
-                }
-                pm_close( ifp );
-                if (h.bscale < 0.0) {
-                    t = dmax;
-                    dmax = dmin;
-                    dmin = t;
-                }
-                ifp = pm_openr( fits_name );
-                read_fits_header( ifp, &h );
-                h.datamin = forcemin ? frmin : dmin * h.bscale + h.bzer;
-                h.datamax = forcemax ? frmax : dmax * h.bscale + h.bzer;
-            }
-        }
-    }
+    if (h.bitpix < 0) {
+        /* samples are floating point, which means the resolution could be
+           anything.  So we just pick a convenient maxval of 255.  We should
+           have a program option to choose the maxval.  Before Netpbm 10.20
+           (January 2004), we did maxval = max - min for floating point as
+           well as integer samples.
+        */
+        maxval = 255;
+    } else
+        maxval = MIN(1, MAX(PNM_OVERALLMAXVAL, datamax - datamin));
 
-
-    maxval = MIN( h.datamax - h.datamin, PNM_OVERALLMAXVAL );
-    scale = 1.0;
-    /* Just in case we have a file with constant value. */
-    if ( h.datamin != h.datamax )
-        scale = maxval / ( h.datamax - h.datamin );
+    if ( datamax - datamin == 0 )
+        scale = 1.0;
+    else
+        scale = maxval / ( datamax - datamin );
 
     /* If printmax option is true, just print and exit. */
     /* For use in shellscripts.  Ex:                    */
@@ -233,12 +291,17 @@ main( argc, argv )
     /*   awk '{min = $1; max = $2}\                     */
     /*         END {print "min=" min; " max=" max}'`    */
     if (printmax) {
-        printf( "%f %f\n", h.datamin, h.datamax);
+        printf( "%f %f\n", datamin, datamax);
         pm_close( ifp );
         pm_close( stdout );
         exit( 0 );
     }
 
+    if (multiplane)
+        format = PPM_FORMAT;
+    else
+        format = PGM_FORMAT;
+
     pnmarray = pnm_allocarray( cols, rows );
 
     switch( PNM_FORMAT_TYPE( format ))
@@ -255,7 +318,7 @@ main( argc, argv )
                 for ( col = 0; col < cols; ++col )
                 {
                     read_val (ifp, h.bitpix, &val);
-                    t = scale * ( val * h.bscale + h.bzer - h.datamin );
+                    t = scale * ( val * h.bscale + h.bzer - datamin );
                     tx = MAX( 0, MIN( t, maxval ) );
                     if ( image == imagenum )
                         PNM_ASSIGN1( pnmarray[row][col], tx );
@@ -274,7 +337,7 @@ main( argc, argv )
                     txv[1] = PPM_GETR( pnmarray[row][col] );
                     txv[2] = PPM_GETG( pnmarray[row][col] );
                     txv[3] = PPM_GETB( pnmarray[row][col] );
-                    t = scale * ( val * h.bscale + h.bzer - h.datamin );
+                    t = scale * ( val * h.bscale + h.bzer - datamin );
                     txv[image] = MAX( 0, MIN( t, maxval ));
                     PPM_ASSIGN( pnmarray[row][col], txv[1], txv[2], txv[3] );
                 }
