@@ -21,6 +21,7 @@
 #include <string.h>
 #include "pnm.h"
 #include "shhopt.h"
+#include "mallocvar.h"
 
 #define GIFMAXVAL 255
 #define MAXCOLORMAPSIZE 256
@@ -555,7 +556,7 @@ struct stack {
 static void 
 initStack(struct stack * const stackP, unsigned int const size) {
 
-    stackP->stack = malloc(size * sizeof(int));
+    MALLOCARRAY(stackP->stack, size);
     if (stackP->stack == NULL)
         pm_error("Unable to allocate %d-word stack.", size);
     stackP->sp = stackP->stack;
@@ -876,7 +877,9 @@ readImageData(FILE * const ifP,
               int    const len, 
               int    const height, 
               gifColorMap  cmap, 
-              bool   const interlace) {
+              bool   const interlace,
+              int    const transparentIndex,
+              bit ** const alphabits) {
 
     unsigned char lzwMinCodeSize;      
     int v;
@@ -906,6 +909,10 @@ readImageData(FILE * const ifP,
         PPM_ASSIGN(xels[ypos][xpos], 
                    cmap[CM_RED][v], cmap[CM_GRN][v], cmap[CM_BLU][v]);
 
+        if (alphabits) 
+            alphabits[ypos][xpos] = (v == transparentIndex) ? 
+                PBM_BLACK : PBM_WHITE;
+
         ++xpos;
         if (xpos == len) {
             xpos = 0;
@@ -986,79 +993,29 @@ writePnm(FILE *outfile, xel ** const xels,
 
 
 static void
-transparencyMessage(int const transparent_index, 
+transparencyMessage(int const transparentIndex, 
                     gifColorMap cmap) {
 /*----------------------------------------------------------------------------
    If user wants verbose output, tell him that the color with index
-   'transparent_index' is supposed to be a transparent background color.
+   'transparentIndex' is supposed to be a transparent background color.
    
-   If transparent_index == -1, tell him there is no transparent background
+   If transparentIndex == -1, tell him there is no transparent background
    color.
 -----------------------------------------------------------------------------*/
     if (verbose) {
-        if (transparent_index == -1)
+        if (transparentIndex == -1)
             pm_message("no transparency");
         else
             pm_message("transparent background color: rgb:%02x/%02x/%02x "
                        "Index %d",
-                       cmap[CM_RED][transparent_index],
-                       cmap[CM_GRN][transparent_index],
-                       cmap[CM_BLU][transparent_index],
-                       transparent_index
+                       cmap[CM_RED][transparentIndex],
+                       cmap[CM_GRN][transparentIndex],
+                       cmap[CM_BLU][transparentIndex],
+                       transparentIndex
                 );
     }
 }
 
-
-
-static void
-outputAlpha(FILE *alpha_file, pixel ** const xels, 
-            const int cols, const int rows, const int transparent_index,
-            unsigned char cmap[3][MAXCOLORMAPSIZE]) {
-/*----------------------------------------------------------------------------
-   Output to file 'alpha_file' (unless it is NULL) the alpha mask for the
-   image 'xels', given that the color whose index in the color map 'cmap' is
-   'transparent_index' is the transparent color.  The image, and thus the
-   alpha mask have dimensions 'cols' by 'rows'.
-
-   transparent_index == -1 means there are no transparent pixels.
------------------------------------------------------------------------------*/
-
-    if (alpha_file) {
-        bit *alpha_row;  /* malloc'ed */
-        xel transparent_color;
-
-        if (transparent_index != -1) 
-            PPM_ASSIGN(transparent_color, 
-                       cmap[CM_RED][transparent_index],
-                       cmap[CM_GRN][transparent_index],
-                       cmap[CM_BLU][transparent_index]
-                );
-        
-        alpha_row = pbm_allocrow(cols);
-
-        pbm_writepbminit(alpha_file, cols, rows, FALSE);
-
-        {
-            int row;
-            for (row = 0; row < rows; row++) {
-                int col;
-                for (col = 0; col < cols; col++) {
-                    if (transparent_index != -1 && 
-                        PNM_EQUAL(xels[row][col], transparent_color))
-                        alpha_row[col] = PBM_BLACK;
-                    else 
-                        alpha_row[col] = PBM_WHITE;
-                }
-                pbm_writepbmrow(alpha_file, alpha_row, cols, FALSE);
-            }
-        }
-        pbm_freerow(alpha_row);
-    }
-}
-
-
-
 static void
 readGifHeader(FILE * const gifFile, struct gifScreen * const gifScreenP) {
 /*----------------------------------------------------------------------------
@@ -1193,6 +1150,10 @@ convertImage(FILE *           const ifP,
     unsigned char buf[16];
     bool useGlobalColormap;
     xel **xels;  /* The image raster, in libpnm format */
+    bit **alphabits;  
+        /* The image alpha mask, in libpbm format.  NULL if we aren't computing
+           an alpha mask.
+        */
     int cols, rows;  /* Dimensions of the image */
     gifColorMap localColorMap;
     int localColorMapSize;
@@ -1211,32 +1172,43 @@ convertImage(FILE *           const ifP,
     if (!xels)
         pm_error("couldn't alloc space for image" );
 
-    if (! useGlobalColormap) {
+    if (alphafile) {
+        alphabits = pbm_allocarray(cols, rows);
+        if (!alphabits)
+            pm_error("couldn't alloc space for alpha image" );
+    } else
+        alphabits = NULL;
+    
+    if (!useGlobalColormap) {
         int hasGray, hasColor;
 
         readColorMap(ifP, localColorMapSize, localColorMap, 
                      &hasGray, &hasColor);
         transparencyMessage(gif89.transparent, localColorMap);
         readImageData(ifP, xels, cols, rows, localColorMap, 
-                      BitSet(buf[8], INTERLACE));
+                      BitSet(buf[8], INTERLACE), gif89.transparent,
+                      alphabits);
         if (!skipIt) {
             writePnm(imageout_file, xels, cols, rows,
                      hasGray, hasColor);
-            outputAlpha(alphafile, xels, cols, rows, 
-                        gif89.transparent, localColorMap);
         }
     } else {
         transparencyMessage(gif89.transparent, gifScreen.ColorMap);
         readImageData(ifP, xels, cols, rows, gifScreen.ColorMap, 
-                      BitSet(buf[8], INTERLACE));
+                      BitSet(buf[8], INTERLACE), gif89.transparent,
+                      alphabits);
         if (!skipIt) {
             writePnm(imageout_file, xels, cols, rows,
                      gifScreen.hasGray, gifScreen.hasColor);
-            outputAlpha(alphafile, xels, cols, rows, 
-                        gif89.transparent, gifScreen.ColorMap);
         }
     }
+
+    if (!skipIt && alphafile && alphabits)
+        pbm_writepbm(alphafile, alphabits, cols, rows, FALSE);
+
     pnm_freearray(xels, rows);
+    if (alphabits)
+        pbm_freearray(alphabits, rows);
 }
 
 
