@@ -62,6 +62,17 @@
 #include "pngtxt.h"
 #include "mallocvar.h"
 
+struct chroma {
+    float wx;
+    float wy;
+    float rx;
+    float ry;
+    float gx;
+    float gy;
+    float bx;
+    float by;
+};
+
 typedef struct cahitem {
     xel color;
     gray alpha;
@@ -93,6 +104,7 @@ typedef struct _jmpbuf_wrapper {
 #  define NONE 0
 #endif
 #define MAXCOLORS 256
+#define MAXPALETTEENTRIES 256
 
 /* PALETTEMAXVAL is the maxval used in a PNG palette */
 #define PALETTEMAXVAL 255
@@ -107,42 +119,11 @@ int main (int argc, char *argv[]);
 #endif
 
 static int verbose = FALSE;
-static int interlace = FALSE;
-static int downscale = FALSE;
-static int transparent = -1;
-static char *transstring;
-static bool alpha = FALSE;
-static char *alpha_file;
-static int background = -1;
-static char *backstring;
-static float gamma = -1.0;
-static int hist = FALSE;
-static float chroma_wx = -1.0;
-static float chroma_wy = -1.0;
-static float chroma_rx = -1.0;
-static float chroma_ry = -1.0;
-static float chroma_gx = -1.0;
-static float chroma_gy = -1.0;
-static float chroma_bx = -1.0;
-static float chroma_by = -1.0;
-static int phys_x = -1.0;
-static int phys_y = -1.0;
-static int phys_unit = -1.0;
-static int text = FALSE;
-static int ztxt = FALSE;
-static char *text_file;
-static int mtime = FALSE;
-static char *date_string;
-static char *time_string;
-static struct tm time_struct;
-static int filter = -1;
-static int compression = -1;
-static int force = FALSE;
+
 static jmpbuf_wrapper pnmtopng_jmpbuf_struct;
 static int errorlevel;
 
 
-
 static png_color_16
 xelToPngColor_16(xel const input, 
                  xelval const maxval, 
@@ -749,6 +730,147 @@ compute_nonalpha_palette(colorhist_vector const chv,
 
 
 
+static void
+computeUnsortedAlphaPalette(FILE *           const ifP,
+                            int              const cols,
+                            int              const rows,
+                            xelval           const maxval,
+                            int              const format,
+                            pm_filepos       const imagepos,
+                            gray **          const alpha_mask,
+                            unsigned int     const maxPaletteEntries,
+                            colorhist_vector const chv,
+                            int              const colors,
+                            gray *                 alphas_of_color[],
+                            unsigned int           alphas_first_index[],
+                            unsigned int           alphas_of_color_cnt[]) {
+/*----------------------------------------------------------------------------
+   Read the image at position 'imagepos' in file *ifP, which is a PNM
+   described by 'cols', 'rows', 'maxval', and 'format'.
+
+   Using the alpha mask 'alpha_mask' and color map 'chv' (of size 'colors')
+   for the image, construct a palette of (color index, alpha) ordered pairs 
+   for the image, as follows.
+
+   The alpha/color palette is the set of all ordered pairs of
+   (color,alpha) in the PNG, including the background color.  The
+   actual palette is an array with up to 'maxPaletteEntries elements.  Each
+   array element contains a color index from the color palette and
+   an alpha value.  All the elements with the same color index are
+   contiguous.  alphas_first_index[x] is the index in the
+   alpha/color palette of the first element that has color index x.
+   alphas_of_color_cnt[x] is the number of elements that have color
+   index x.  alphas_of_color[x][y] is the yth alpha value that
+   appears with color index x (in order of appearance).
+   alpha_color_pair_count is the total number of elements, i.e. the
+   total number of combinations color and alpha.
+-----------------------------------------------------------------------------*/
+    colorhash_table cht;
+    int color_index;
+    int row;
+    xel * xelrow;
+
+    cht = ppm_colorhisttocolorhash (chv, colors);
+
+    for (color_index = 0 ; color_index < colors + 1 ; ++color_index) {
+        /* TODO: It sure would be nice if we didn't have to allocate
+           256 words here for what is normally only 0 or 1 different
+           alpha values!  Maybe we should do some sophisticated reallocation.
+        */
+        MALLOCARRAY(alphas_of_color[color_index], maxPaletteEntries);
+        if (alphas_of_color[color_index] == NULL)
+            pm_error ("out of memory allocating alpha/palette entries");
+        alphas_of_color_cnt[color_index] = 0;
+    }
+ 
+    pm_seek2(ifP, &imagepos, sizeof(imagepos));
+
+    xelrow = pnm_allocrow(cols);
+
+    for (row = 0 ; row < rows ; ++row) {
+        int col;
+        pnm_readpnmrow(ifP, xelrow, cols, maxval, format);
+        pnm_promoteformatrow(xelrow, cols, maxval, format, maxval, PPM_TYPE);
+        for (col = 0 ; col < cols ; ++col) {
+            int i;
+            int const color = ppm_lookupcolor(cht, &xelrow[col]);
+            for (i = 0 ; i < alphas_of_color_cnt[color] ; ++i) {
+                if (alpha_mask[row][col] == alphas_of_color[color][i])
+                    break;
+            }
+            if (i == alphas_of_color_cnt[color]) {
+                alphas_of_color[color][i] = alpha_mask[row][col];
+                alphas_of_color_cnt[color]++;
+            }
+        }
+    }
+    {
+        int i;
+        alphas_first_index[0] = 0;
+        for (i = 1 ; i < colors ; i++)
+            alphas_first_index[i] = alphas_first_index[i-1] +
+                alphas_of_color_cnt[i-1];
+    }
+    pnm_freerow(xelrow);
+    ppm_freecolorhash(cht);
+}
+
+
+
+static void
+sortAlphaPalette(gray *alphas_of_color[],
+                 unsigned int alphas_first_index[],
+                 unsigned int alphas_of_color_cnt[],
+                 unsigned int const colors,
+                 unsigned int mapping[],
+                 unsigned int * const transSizeP) {
+/*----------------------------------------------------------------------------
+   Remap the palette indices so opaque entries are last.
+
+   alphas_of_color[], alphas_first_index[], and alphas_of_color_cnt[]
+   describe an unsorted PNG (alpha/color) palette.  We generate
+   mapping[] such that mapping[x] is the index into the sorted PNG
+   palette of the alpha/color pair whose index is x in the unsorted
+   PNG palette.  This mapping sorts the palette so that opaque entries
+   are last.
+-----------------------------------------------------------------------------*/
+    unsigned int bot_idx;
+    unsigned int top_idx;
+    unsigned int colorIndex;
+    
+    /* We start one index at the bottom of the palette index range
+       and another at the top.  We run through the unsorted palette,
+       and when we see an opaque entry, we map it to the current top
+       cursor and bump it down.  When we see a non-opaque entry, we map 
+       it to the current bottom cursor and bump it up.  Because the input
+       and output palettes are the same size, the two cursors should meet
+       right when we process the last entry of the unsorted palette.
+    */    
+    bot_idx = 0;
+    top_idx = alphas_first_index[colors-1] + alphas_of_color_cnt[colors-1] - 1;
+    
+    for (colorIndex = 0;  colorIndex < colors;  ++colorIndex) {
+        unsigned int j;
+        for (j = 0; j < alphas_of_color_cnt[colorIndex]; ++j) {
+            unsigned int const paletteIndex = 
+                alphas_first_index[colorIndex] + j;
+            if (alphas_of_color[colorIndex][j] == PALETTEOPAQUE)
+                mapping[paletteIndex] = top_idx--;
+                else
+                    mapping[paletteIndex] = bot_idx++;
+        }
+    }
+    /* indices should have just crossed paths */
+    if (bot_idx != top_idx + 1) {
+        pm_error ("internal inconsistency: "
+                  "remapped bot_idx = %u, top_idx = %u",
+                  bot_idx, top_idx);
+    }
+    *transSizeP = bot_idx;
+}
+
+
+
 static void
 compute_alpha_palette(FILE *         const ifP, 
                       int            const cols,
@@ -777,135 +899,68 @@ compute_alpha_palette(FILE *         const ifP,
    The palette is sorted so that the opaque entries are last, and we return
    *transSizeP as the number of non-opaque entries.
 
-   palette[] and trans[] are allocated by the caller to at least MAXCOLORS
-   elements.
+   palette[] and trans[] are allocated by the caller to at least 
+   MAXPALETTEENTRIES elements.
 
-   If there are more than MAXCOLORS color/alpha pairs in the image, don't
-   return any palette information -- just return *tooBigP == TRUE.
+   If there are more than MAXPALETTEENTRIES color/alpha pairs in the image, 
+   don't return any palette information -- just return *tooBigP == TRUE.
 -----------------------------------------------------------------------------*/
     colorhist_vector chv;
     int colors;
-    colorhash_table cht;
-    int color_index;
-    int row;
-    xel * xelrow;
-
-    /* The following data structures describe the alpha/color palette.
-       The alpha/color palette is the set of all ordered pairs of
-       (color,alpha) in the PNG, including the background color.  The
-       actual palette is an array with up to MAXCOLORS elements.  Each
-       array element contains a color index from the color palette and
-       an alpha value.  All the elements with the same color index are
-       contiguous.  alphas_first_index[x] is the index in the
-       alpha/color palette of the first element that has color index x.
-       alphas_of_color_cnt[x] is the number of elements that have color
-       index x.  alphas_of_color[x][y] is the yth alpha value that
-       appears with color index x (in order of appearance).
-       alpha_color_pair_count is the total number of elements, i.e. the
-       total number of combinations color and alpha.
-    */
-    gray *alphas_of_color[MAXCOLORS];
-    unsigned int alphas_first_index[MAXCOLORS];
-    unsigned int alphas_of_color_cnt[MAXCOLORS];
 
+    gray *alphas_of_color[MAXPALETTEENTRIES];
+    unsigned int alphas_first_index[MAXPALETTEENTRIES];
+    unsigned int alphas_of_color_cnt[MAXPALETTEENTRIES];
+ 
     getChv(ifP, imagepos, cols, rows, maxval, format, MAXCOLORS, 
            &chv, &colors);
-    cht = ppm_colorhisttocolorhash (chv, colors);
 
-    for (color_index = 0 ; color_index < colors + 1 ; ++color_index) {
-        MALLOCARRAY(alphas_of_color[color_index], MAXCOLORS);
-        if (alphas_of_color[color_index] == NULL)
-            pm_error ("out of memory allocating alpha/palette entries");
-        alphas_of_color_cnt[color_index] = 0;
-    }
- 
-    pm_seek2(ifP, &imagepos, sizeof(imagepos));
+    computeUnsortedAlphaPalette(ifP, cols, rows, maxval, format, imagepos,
+                                alpha_mask, MAXPALETTEENTRIES, chv, colors,
+                                alphas_of_color,
+                                alphas_first_index,
+                                alphas_of_color_cnt);
 
-    xelrow = pnm_allocrow(cols);
-
-    for (row = 0 ; row < rows ; ++row) {
-        int col;
-        pnm_readpnmrow(ifP, xelrow, cols, maxval, format);
-        pnm_promoteformatrow(xelrow, cols, maxval, format, maxval, PPM_TYPE);
-        for (col = 0 ; col < cols ; ++col) {
-            int i;
-            int const color = ppm_lookupcolor(cht, &xelrow[col]);
-            for (i = 0 ; i < alphas_of_color_cnt[color] ; i++) {
-                if (alpha_mask[row][col] == alphas_of_color[color][i])
-                    break;
-            }
-            if (i == alphas_of_color_cnt[color]) {
-                alphas_of_color[color][i] = alpha_mask[row][col];
-                alphas_of_color_cnt[color]++;
-            }
-        }
-    }
-    pnm_freerow(xelrow);
-
-    {
-        int i;
-        alphas_first_index[0] = 0;
-        for (i = 1 ; i < colors ; i++)
-            alphas_first_index[i] = alphas_first_index[i-1] +
-                alphas_of_color_cnt[i-1];
-    }
     *paletteSizeP = 
         alphas_first_index[colors-1] + alphas_of_color_cnt[colors-1];
-    if (*paletteSizeP > MAXCOLORS) {
+    if (*paletteSizeP > MAXPALETTEENTRIES) {
         *tooBigP = TRUE;
     } else {
-        int mapping[MAXCOLORS];
+        unsigned int mapping[MAXPALETTEENTRIES];
             /* Sorting of the alpha/color palette.  mapping[x] is the
                index into the sorted PNG palette of the alpha/color
                pair whose index is x in the unsorted PNG palette.
                This mapping sorts the palette so that opaque entries
                are last.  
             */
-        unsigned int bot_idx;
-        unsigned int top_idx;
-        unsigned int colorIndex;
 
         *tooBigP = FALSE;
 
-        bot_idx = 0;
-        top_idx = alphas_first_index[colors] + alphas_of_color_cnt[colors] - 1;
+        /* Make the opaque palette entries last */
+        sortAlphaPalette(alphas_of_color, alphas_first_index,
+                         alphas_of_color_cnt, colors,
+                         mapping, transSizeP);
 
-        /* remap palette indices so opaque entries are last */
-        for (colorIndex = 0;  colorIndex < colors;  ++colorIndex) {
-            unsigned int j;
-            for (j = 0; j < alphas_of_color_cnt[colorIndex]; ++j) {
-                unsigned int const paletteIndex = 
-                    alphas_first_index[colorIndex] + j;
-                if (alphas_of_color[colorIndex][j] == PALETTEOPAQUE)
-                    mapping[paletteIndex] = top_idx--;
-                else
-                    mapping[paletteIndex] = bot_idx++;
-            }
-        }
-        /* indices should have just crossed paths */
-        if (bot_idx != top_idx + 1) {
-            pm_error ("internal inconsistency: "
-                      "remapped bot_idx = %d, top_idx = %d",
-                      bot_idx, top_idx);
-        }
-        for (colorIndex = 0; colorIndex < colors; ++colorIndex) {
-            unsigned int j;
-            for (j = 0; j < alphas_of_color_cnt[colorIndex]; ++j) {
-                unsigned int const paletteIndex = 
-                    alphas_first_index[colorIndex] + j;
-                palette_pnm[mapping[paletteIndex]] = chv[colorIndex].color;
-                trans_pnm[mapping[paletteIndex]] = 
+        {
+            unsigned int colorIndex;
+
+            for (colorIndex = 0; colorIndex < colors; ++colorIndex) {
+                unsigned int j;
+                for (j = 0; j < alphas_of_color_cnt[colorIndex]; ++j) {
+                    unsigned int const paletteIndex = 
+                        alphas_first_index[colorIndex] + j;
+                    palette_pnm[mapping[paletteIndex]] = chv[colorIndex].color;
+                    trans_pnm[mapping[paletteIndex]] = 
                     alphas_of_color[colorIndex][j];
+                }
             }
         }
-        *transSizeP = bot_idx;
     }
     { 
-        unsigned int color_index;
-        for (color_index = 0; color_index < colors + 1; ++color_index)
-            free(alphas_of_color[color_index]);
+        unsigned int colorIndex;
+        for (colorIndex = 0; colorIndex < colors + 1; ++colorIndex)
+            free(alphas_of_color[colorIndex]);
     }
-    ppm_freecolorhash(cht);
 } 
 
 
@@ -1238,10 +1293,35 @@ createPngPalette(pixel              palette_pnm[],
 
 
 static int 
-convertpnm (FILE * const ifp, 
-            FILE * const afp, 
-            FILE * const pfp, 
-            FILE * const tfp) {
+convertpnm(FILE * const ifp, 
+           FILE * const afp, 
+           FILE * const pfp, 
+           FILE * const tfp,
+           float  const gamma,
+           int    const interlace,
+           int    const downscale,
+           bool   const transparent_opt,
+           char * const transstring,
+           bool   const alpha_opt,
+           char * const alpha_file,
+           int    const background,
+           char * const backstring,
+           int    const hist,
+           struct chroma const chroma,
+           int    const phys_x,
+           int    const phys_y,
+           int    const phys_unit,
+           int    const text,
+           int    const ztxt,
+           char * const text_file,
+           int    const mtime,
+           char * const date_string,
+           char * const time_string,
+           int    const filter,
+           int    const compression,
+           int    const force
+    ) {
+
   xel p;
   int rows, cols, format;
   xelval maxval;
@@ -1258,6 +1338,9 @@ convertpnm (FILE * const ifp,
     /* boolean: the user wants only the exact color he specified to be
        transparent; not just something close to it.
     */
+  int transparent;
+  bool alpha;
+    /* There will be an alpha mask */
   unsigned int pnm_meaningful_bits;
   pixel backcolor;
       /* The background color, with maxval equal to that of the input
@@ -1369,7 +1452,7 @@ convertpnm (FILE * const ifp,
   else if (pnm_type == PPM_TYPE)
     maxmaxval = PPM_OVERALLMAXVAL;
 
-  if (transparent > 0) {  /* -1 or 1 are the only possibilities so far */
+  if (transparent_opt) {
     char * transstring2;  
       /* Same as transstring, but with possible leading '=' removed */
     if (transstring[0] == '=') {
@@ -1386,7 +1469,7 @@ convertpnm (FILE * const ifp,
     PPM_DEPTH (transcolor, ppm_parsecolor(transstring2, maxmaxval),
                maxmaxval, maxval);
   }
-  if (alpha) {
+  if (alpha_opt) {
     pixel alpha_transcolor;
 
     if (verbose)
@@ -1409,15 +1492,18 @@ convertpnm (FILE * const ifp,
       transparent = 2;
       transcolor = alpha_transcolor;
     } else {
+      alpha = TRUE;
       transparent = -1;
     }
-  } else
+  } else {
       /* Though there's no alpha_mask, we still need an alpha_maxval for
          use with trans[], which can have stuff in it if the user specified
          a transparent color.
       */
+      alpha = FALSE;
       alpha_maxval = 255;
-
+      transparent = transparent_opt ? 1 : -1;
+  }
   /* gcc 2.7.0 -fomit-frame-pointer causes stack corruption here */
   if (background > -1) 
       PPM_DEPTH(backcolor, ppm_parsecolor (backstring, maxmaxval), 
@@ -1581,16 +1667,16 @@ convertpnm (FILE * const ifp,
   }
 
   /* cHRM chunk */
-  if (chroma_wx != -1.0) {
+  if (chroma.wx != -1.0) {
     info_ptr->valid |= PNG_INFO_cHRM;
-    info_ptr->x_white = chroma_wx;
-    info_ptr->y_white = chroma_wy;
-    info_ptr->x_red = chroma_rx;
-    info_ptr->y_red = chroma_ry;
-    info_ptr->x_green = chroma_gx;
-    info_ptr->y_green = chroma_gy;
-    info_ptr->x_blue = chroma_bx;
-    info_ptr->y_blue = chroma_by;
+    info_ptr->x_white = chroma.wx;
+    info_ptr->y_white = chroma.wy;
+    info_ptr->x_red = chroma.rx;
+    info_ptr->y_red = chroma.ry;
+    info_ptr->x_green = chroma.gx;
+    info_ptr->y_green = chroma.gy;
+    info_ptr->x_blue = chroma.bx;
+    info_ptr->y_blue = chroma.by;
   }
 
   /* pHYS chunk */
@@ -1729,6 +1815,7 @@ convertpnm (FILE * const ifp,
 
   /* tIME chunk */
   if (mtime) {
+    struct tm time_struct;
     info_ptr->valid |= PNG_INFO_tIME;
     sscanf (date_string, "%d-%d-%d", &time_struct.tm_year,
                                      &time_struct.tm_mon,
@@ -1900,6 +1987,30 @@ displayVersion() {
 int 
 main(int argc, char *argv[]) {
 
+  float gamma = -1.0;
+  int interlace = FALSE;
+  int downscale = FALSE;
+  bool transparent = FALSE;
+  char *transstring;
+  bool alpha = FALSE;
+  char *alpha_file;
+  int background = -1;
+  char *backstring;
+  int hist = FALSE;
+  struct chroma chroma = {-1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0};
+  int phys_x = -1.0;
+  int phys_y = -1.0;
+  int phys_unit = -1.0;
+  int text = FALSE;
+  int ztxt = FALSE;
+  char *text_file;
+  int mtime = FALSE;
+  char *date_string;
+  char *time_string;
+  int filter = -1;
+  int compression = -1;
+  int force = FALSE;
+
   FILE *ifp, *tfp, *afp, *pfp;
   char *palette_file;
       /* Name of user-supplied palette, file; NULL if none */
@@ -1927,7 +2038,7 @@ main(int argc, char *argv[]) {
       interlace = TRUE;
     } else
     if (pm_keymatch (argv[argn], "-alpha", 2)) {
-      if (transparent > 0)
+      if (transparent)
         pm_error ("-alpha and -transparent are mutually exclusive");
       alpha = TRUE;
       if (++argn < argc)
@@ -1938,7 +2049,7 @@ main(int argc, char *argv[]) {
     if (pm_keymatch (argv[argn], "-transparent", 3)) {
       if (alpha)
         pm_error ("-alpha and -transparent are mutually exclusive");
-      transparent = 1;
+      transparent = TRUE;
       if (++argn < argc)
         transstring = argv[argn];
       else
@@ -1962,35 +2073,35 @@ main(int argc, char *argv[]) {
     } else
     if (pm_keymatch (argv[argn], "-chroma", 3)) {
       if (++argn < argc)
-        sscanf (argv[argn], "%f", &chroma_wx);
+        sscanf (argv[argn], "%f", &chroma.wx);
       else
         pm_usage (usage);
       if (++argn < argc)
-        sscanf (argv[argn], "%f", &chroma_wy);
+        sscanf (argv[argn], "%f", &chroma.wy);
       else
         pm_usage (usage);
       if (++argn < argc)
-        sscanf (argv[argn], "%f", &chroma_rx);
+        sscanf (argv[argn], "%f", &chroma.rx);
       else
         pm_usage (usage);
       if (++argn < argc)
-        sscanf (argv[argn], "%f", &chroma_ry);
+        sscanf (argv[argn], "%f", &chroma.ry);
       else
         pm_usage (usage);
       if (++argn < argc)
-        sscanf (argv[argn], "%f", &chroma_gx);
+        sscanf (argv[argn], "%f", &chroma.gx);
       else
         pm_usage (usage);
       if (++argn < argc)
-        sscanf (argv[argn], "%f", &chroma_gy);
+        sscanf (argv[argn], "%f", &chroma.gy);
       else
         pm_usage (usage);
       if (++argn < argc)
-        sscanf (argv[argn], "%f", &chroma_bx);
+        sscanf (argv[argn], "%f", &chroma.bx);
       else
         pm_usage (usage);
       if (++argn < argc)
-        sscanf (argv[argn], "%f", &chroma_by);
+        sscanf (argv[argn], "%f", &chroma.by);
       else
         pm_usage (usage);
     } else
@@ -2103,7 +2214,13 @@ main(int argc, char *argv[]) {
   else
     tfp = NULL;
 
-  errorlevel = convertpnm(ifp, afp, pfp, tfp);
+  errorlevel = convertpnm(ifp, afp, pfp, tfp,
+                          gamma, interlace, downscale, transparent,
+                          transstring, alpha, alpha_file, background,
+                          backstring, hist, chroma, phys_x, phys_y,
+                          phys_unit, text, ztxt, text_file, mtime,
+                          date_string, time_string, filter,
+                          compression, force);
 
   if (afp)
     pm_closer (afp);
