@@ -52,6 +52,7 @@
 #include <string.h>
 #include "pnm.h"
 #include "shhopt.h"
+#include "mallocvar.h"
 
 #ifdef VMS
 #ifdef SYSV
@@ -196,19 +197,6 @@ read_directory(TIFF * const tif,
         *planarconfig_p = PLANARCONFIG_CONTIG;
     }
 
-    switch (*spp_p) {
-    case 1:
-    case 3:
-    case 4:
-        break;
-    default:
-        pm_error(
-            "This program can only handle 1-channel gray scale "
-            "or 1- or 3-channel color.  The input Tiff file has %d channels "
-            "(samples per pixel)",
-            *spp_p);
-    }
-
     switch(*planarconfig_p) {
     case PLANARCONFIG_CONTIG:
         break;
@@ -405,10 +393,13 @@ analyze_image_type(TIFF * const tif, unsigned short const bps,
     } else {
         /* How come we don't deal with the photometric for the monochrome 
            case (make sure it's one we know)?  -Bryan 00.03.04
-           */
+        */
         switch ( photomet ) {
         case PHOTOMETRIC_MINISBLACK:
         case PHOTOMETRIC_MINISWHITE:
+            if (spp != 1)
+                pm_error("This grayscale image has %d samples per pixel.  "
+                         "We understand only 1.", spp);
             grayscale = TRUE;
             *maxvalP = pm_bitstomaxval(MIN(bps,16));
             if (headerdump)
@@ -429,6 +420,10 @@ analyze_image_type(TIFF * const tif, unsigned short const bps,
             if (headerdump)
                 pm_message("colormapped");
 
+            if (spp != 1)
+                pm_error("This paletted image has %d samples per pixel.  "
+                         "We understand only 1.", spp);
+
             if (!TIFFGetField(tif, TIFFTAG_COLORMAP, 
                               &redcolormap, &greencolormap, &bluecolormap))
                 pm_error("error getting colormaps");
@@ -472,6 +467,11 @@ analyze_image_type(TIFF * const tif, unsigned short const bps,
             if (headerdump)
                 pm_message("RGB truecolor");
             grayscale = FALSE;
+
+            if (spp != 3 && spp != 4)
+                pm_error("This RGB image has %d samples per pixel.  "
+                         "We understand only 3 or 4.", spp);
+
             *maxvalP = (1 << bps) - 1;
             break;
 
@@ -717,7 +717,7 @@ convertRasterByRows(FILE *         const imageoutFile,
     if (scanbuf == NULL)
         pm_error("can't allocate memory for scanline buffer");
 
-    samplebuf = (unsigned int *) malloc(cols * sizeof(samplebuf[0]) * spp);
+    MALLOCARRAY(samplebuf, cols * spp);
     if (samplebuf == NULL)
         pm_error ("can't allocate memory for row buffer");
 
@@ -854,13 +854,12 @@ convertRasterInMemory(FILE *         const imageoutFile,
             pm_message(emsg);
             *statusP = CONV_UNABLE;
         } else {
-            unsigned int const rasterSize = cols * rows * sizeof(uint32);
             uint32* raster ;
 
-            raster = (uint32 *) malloc(rasterSize);
+            MALLOCARRAY(raster, cols * rows);
             if (raster == NULL) {
-                pm_message("Unable to allocate %u bytes of memory to hold "
-                           "raster.", rasterSize);
+                pm_message("Unable to allocate space for a raster of %u "
+                           "pixels.", cols * rows);
                 *statusP = CONV_OOM;
             } else {
                 int const stopOnErrorFalse = FALSE;
