@@ -4,9 +4,6 @@
  * ppmtobmp.c - Converts from a PPM file to a Microsoft Windows or OS/2
  * .BMP file.
  *
- * The current implementation is probably not complete, but it works for
- * me.  I welcome feedback.
- *
  * Copyright (C) 1992 by David W. Sanderson.
  *
  * Permission to use, copy, modify, and distribute this software and its
@@ -59,8 +56,6 @@ enum colortype {TRUECOLOR, PALETTE};
  * Utilities
  */
 
-static char     er_write[] = "stdout: write error";
-
 static struct cmdline_info {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
@@ -133,34 +128,36 @@ parse_command_line(int argc, char ** argv,
 
 
 static void
-PutByte(FILE *fp, char v)
-{
-        if (putc(v, fp) == EOF)
-        {
-                pm_error(er_write);
-        }
+PutByte(FILE * const fp, unsigned char const v) {
+    if (putc(v, fp) == EOF) 
+        pm_error("Write of a byte to a file failed.");
+
+    /* Note:  a Solaris/SPARC user reported on 2003.09.29 that the above
+       putc() returned EOF when a former version of this code declared
+       v as "char" instead of "unsigned char".  This was apparently due
+       to a bug in his C library that caused 255 to look like -1 at some
+       critical internal point.
+    */
 }
 
-static void
-PutShort(FILE *fp, short v)
-{
-        if (pm_writelittleshort(fp, v) == -1)
-        {
-                pm_error(er_write);
-        }
-}
+
 
 static void
-PutLong(fp, v)
-        FILE           *fp;
-        long            v;
-{
-        if (pm_writelittlelong(fp, v) == -1)
-        {
-                pm_error(er_write);
-        }
+PutShort(FILE * const fp, short const v) {
+    if (pm_writelittleshort(fp, v) == -1) 
+        pm_error("Write of a halfword to a file failed.");
 }
 
+
+
+static void
+PutLong(FILE * const fp, long const v) {
+    if (pm_writelittlelong(fp, v) == -1)
+        pm_error("Write of a word to a file failed.");
+}
+
+
+
 /*
  * BMP writing
  */
@@ -357,13 +354,13 @@ BMPwriterow_palette(FILE *fp, const pixel * const row,
         return nbyte;
 }
 
-/*
- * returns the number of bytes written, or -1 on error.
- */
+
+
 static int
 BMPwriterow_truecolor(FILE *        const fp, 
                       const pixel * const row, 
-                      unsigned long const cols) {
+                      unsigned long const cols,
+                      pixval        const maxval) {
 /*----------------------------------------------------------------------------
   Write a row of a truecolor BMP image to the file 'fp'.  The row is 
   'row', which is 'cols' columns long.
@@ -385,9 +382,10 @@ BMPwriterow_truecolor(FILE *        const fp,
         
     nbyte = 0;  /* initial value */
     for (col = 0; col < cols; col++) {
-        PutByte(fp, PPM_GETB(row[col]));
-        PutByte(fp, PPM_GETG(row[col]));
-        PutByte(fp, PPM_GETR(row[col]));
+        /* We scale to the BMP maxval, which is always 255. */
+        PutByte(fp, PPM_GETB(row[col]) * 255 / maxval);
+        PutByte(fp, PPM_GETG(row[col]) * 255 / maxval);
+        PutByte(fp, PPM_GETR(row[col]) * 255 / maxval);
         nbyte += 3;
     }
 
@@ -406,10 +404,14 @@ BMPwriterow_truecolor(FILE *        const fp,
  * returns the number of bytes written, or -1 on error.
  */
 static int
-BMPwritebits(FILE *fp, const unsigned long cx, const unsigned long cy, 
-             const enum colortype colortype,
-             const unsigned short cBitCount, 
-             const pixel ** const pixels, const colorhash_table cht)
+BMPwritebits(FILE *          const fp, 
+             unsigned long   const cx, 
+             unsigned long   const cy, 
+             enum colortype  const colortype,
+             unsigned short  const cBitCount, 
+             const pixel **  const pixels, 
+             pixval          const maxval,
+             colorhash_table const cht)
 {
         int             nbyte = 0;
         long            y;
@@ -431,7 +433,7 @@ BMPwritebits(FILE *fp, const unsigned long cx, const unsigned long cy,
                     rc = BMPwriterow_palette(fp, pixels[y], cx, 
                                              cBitCount, cht);
                 else 
-                    rc = BMPwriterow_truecolor(fp, pixels[y], cx);
+                    rc = BMPwriterow_truecolor(fp, pixels[y], cx, maxval);
 
                 if(rc == -1)
                 {
@@ -456,13 +458,19 @@ BMPwritebits(FILE *fp, const unsigned long cx, const unsigned long cy,
  * arrays are undefined.
  */
 static void
-BMPEncode(FILE *fp, const int class, const enum colortype colortype,
-          const int bpp,
-          const int x, const int y, 
-          const pixel ** const pixels, const int colors, 
-          const colorhash_table cht, 
-          const unsigned char R[], const unsigned char G[], 
-          const unsigned char B[])
+BMPEncode(FILE *          const fp, 
+          int             const class, 
+          enum colortype  const colortype,
+          int             const bpp,
+          int             const x, 
+          int             const y, 
+          const pixel **  const pixels, 
+          pixval          const maxval,
+          int             const colors, 
+          colorhash_table const cht, 
+          unsigned char   const R[], 
+          unsigned char   const G[], 
+          unsigned char   const B[])
 {
         unsigned long   nbyte = 0;
 
@@ -484,7 +492,7 @@ BMPEncode(FILE *fp, const int class, const enum colortype colortype,
                 pm_error(er_internal, "BMPEncode 1");
         }
 
-        nbyte += BMPwritebits(fp, x, y, colortype, bpp, pixels, cht);
+        nbyte += BMPwritebits(fp, x, y, colortype, bpp, pixels, maxval, cht);
         if(nbyte != BMPlenfile(class, bpp, -1, x, y))
         {
                 pm_error(er_internal, "BMPEncode 2");
@@ -620,7 +628,7 @@ main(int argc, char **argv)
                              &bits_per_pixel);
 
         BMPEncode(stdout, cmdline.class, colortype, bits_per_pixel,
-                  cols, rows, (const pixel**)pixels, colors, cht,
+                  cols, rows, (const pixel**)pixels, maxval, colors, cht,
                   Red, Green, Blue);
 
         if (cht) 
