@@ -34,6 +34,7 @@
 #include "ppm.h"
 #include "shhopt.h"
 #include "nstring.h"
+#include "mallocvar.h"
 
 #define MAX_LINE 2048
   /* The maximum size XPM input line we can handle. */
@@ -420,7 +421,9 @@ readXpm3Header(FILE * const stream, int * const widthP, int * const heightP,
     } else {
         /* Set up lookup table (see above) */
         *colorsP = ppm_allocrow(*ncolorsP);
-        *ptabP = (int *) malloc(*ncolorsP * sizeof(int));
+        MALLOCARRAY(*ptabP, *ncolorsP);
+        if (*ptabP == NULL)
+            pm_error("Unable to allocate memory for %d colors", *ncolorsP);
     }
     
     { 
@@ -532,7 +535,9 @@ readXpm1Header(FILE * const stream, int * const widthP, int * const heightP,
     } else {
         /* Over two chars per pixel, we fall back on linear search. */
         *colorsP = ppm_allocrow(*ncolorsP);
-        *ptabP = (int *) malloc(*ncolorsP * sizeof(int));
+        MALLOCARRAY(*ptabP, *ncolorsP);
+        if (*ptabP == NULL)
+            pm_error("Unable to allocate memory for %d colors", *ncolorsP);
     }
 
     /* Read color table. */
@@ -679,7 +684,7 @@ ReadXPMFile(FILE * const stream, int * const widthP, int * const heightP,
         *transparentP = -1;  /* No transparency in version 1 */
     }
     totalpixels = *widthP * *heightP;
-    *dataP = (int *) malloc(totalpixels * sizeof(int));
+    MALLOCARRAY(*dataP, totalpixels);
     if (*dataP == NULL)
         pm_error("Could not get %d bytes of memory for image", totalpixels);
     cursor = *dataP;
