@@ -1,4 +1,9 @@
-/* pnminterp - scale up portable anymap by interpolating between pixels.
+/* pamstretch - scale up portable anymap by interpolating between pixels.
+ * 
+ * This program is based on 'pnminterp' by Russell Marks, rename
+ * pnmstretch for inclusion in Netpbm, then rewritten and renamed to
+ * pamstretch by Bryan Henderson in December 2001.
+ *
  * Copyright (C) 1998,2000 Russell Marks.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -13,8 +18,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- */
+ * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #include <stdio.h>
 #include <string.h>
@@ -377,9 +381,9 @@ main(int argc,char *argv[]) {
     if (PNM_FORMAT_TYPE(inpam.format) == PBM_TYPE) {
         outpam.format = PGM_TYPE;
         /* usual filter message when reading PBM but writing PGM: */
-        pm_message("pamstretch: promoting from PBM to PGM");
+        pm_message("promoting from PBM to PGM");
     } else {
-        outpam.format = PPM_TYPE;
+        outpam.format = inpam.format;
     }
     {
         unsigned int const dropped = cmdline.edge_mode == EDGE_DROP ? 1 : 0;
@@ -390,7 +394,7 @@ main(int argc,char *argv[]) {
         pnm_writepaminit(&outpam);
     }
 
-    createBlackTuple(&outpam, &blackTuple);
+    pnm_createBlackTuple(&outpam, &blackTuple);
 
     stretch(&inpam, &outpam, 
             cmdline.xscale, cmdline.yscale, cmdline.edge_mode);
