@@ -14,6 +14,7 @@
 #include <ctype.h>
 #include "pnm.h"
 #include "shhopt.h"
+#include "mallocvar.h"
 
 enum transferFunction {XF_EXP, XF_CIERAMP, XF_SRGBRAMP};
 
@@ -144,7 +145,7 @@ buildCie709Gamma(xelval table[], xelval const maxval,
        The exponential is slightly stretched and translated, though,
        unlike the popular pure exponential gamma transfer function.
     */
-    xelval const linearCutoff = (xelval) maxval * 0.018 + 0.5;
+    xelval const linearCutoff = (xelval) (maxval * 0.018 + 0.5);
     double const linearExpansion = 
         (1.099 * pow(0.018, oneOverGamma) - 0.099) / 0.018;
 
@@ -178,7 +179,7 @@ buildCie709GammaInverse(xelval table[], xelval const maxval,
        The exponential is slightly stretched and translated, though,
        unlike the popular pure exponential gamma transfer function.
     */
-    xelval const linearCutoff = (xelval) maxval * 0.018 + 0.5;
+    xelval const linearCutoff = (xelval) (maxval * 0.018 + 0.5);
     double const linearExpansion = 
         (1.099 * pow(0.018, oneOverGamma) - 0.099) / 0.018;
 
@@ -273,9 +274,9 @@ createGammaTables(bool const ungamma,
                   xelval **rtableP, xelval **gtableP, xelval **btableP) {
 
     /* Allocate space for the tables. */
-    *rtableP = (xelval*) malloc( (maxval+1) * sizeof(xelval) );
-    *gtableP = (xelval*) malloc( (maxval+1) * sizeof(xelval) );
-    *btableP = (xelval*) malloc( (maxval+1) * sizeof(xelval) );
+    MALLOCARRAY(*rtableP, maxval+1);
+    MALLOCARRAY(*gtableP, maxval+1);
+    MALLOCARRAY(*btableP, maxval+1);
     if (*rtableP == NULL || *gtableP == NULL || *btableP == NULL)
         pm_error( "out of memory" );
 
