@@ -54,6 +54,7 @@
 
 #include <math.h>
 #include "ppm.h"
+#include "mallocvar.h"
 
 #ifndef M_PI
 #define M_PI    3.14159265358979323846
@@ -435,18 +436,14 @@ static void genplanet(a, n)
                      (pixval) RGBQuant, FALSE);
 
     if (!stars) {
-        u = (double *) malloc((unsigned int) (screenxsize * sizeof(double)));
-        u1 = (double *) malloc((unsigned int) (screenxsize * sizeof(double)));
-        bxf = (unsigned int *) malloc((unsigned int) screenxsize *
-                                      sizeof(unsigned int));
-        bxc = (unsigned int *) malloc((unsigned int) screenxsize *
-                                      sizeof(unsigned int));
+        MALLOCARRAY(u,   screenxsize);
+        MALLOCARRAY(u1,  screenxsize);
+        MALLOCARRAY(bxf, screenxsize);
+        MALLOCARRAY(bxc, screenxsize);
 
-        if (u == (double *) 0 || u1 == (double *) 0 ||
-            bxf == (unsigned int *) 0 || bxc == (unsigned int *) 0) {
+        if (u == NULL || u1 == NULL || bxf == NULL || bxc == NULL) 
             pm_error("Cannot allocate %d element interpolation tables.",
                      screenxsize);
-        }
 
         /* Compute incident light direction vector. */
 
@@ -479,7 +476,7 @@ static void genplanet(a, n)
         /* Prescale the grid points into intensities. */
 
         cp = (unsigned char *) malloc(n * n);
-        if (cp == (unsigned char *) 0)
+        if (cp == NULL)
             return;
         ap = cp;
         for (i = 0; i < n; i++) {
