@@ -1,7 +1,7 @@
 /*----------------------------------------------------------------------------
                                   libpam.c
 ------------------------------------------------------------------------------
-   These are the library functions, which belong in the libpnm library,
+   These are the library functions, which belong in the libnetpbm library,
    that deal with the PAM (Portable Arbitrary Format) image format.
 -----------------------------------------------------------------------------*/
 
@@ -180,7 +180,8 @@ pnm_scaletuplerow(const struct pam * const pamP,
 
  
 void
-createBlackTuple(const struct pam * const pamP, tuple * const blackTupleP) {
+pnm_createBlackTuple(const struct pam * const pamP, 
+                     tuple *            const blackTupleP) {
 /*----------------------------------------------------------------------------
    Create a "black" tuple.  By that we mean a tuple all of whose elements
    are zero.  If it's an RGB, grayscale, or b&w pixel, that means it's black.
@@ -204,6 +205,20 @@ createBlackTuple(const struct pam * const pamP, tuple * const blackTupleP) {
 
 
 
+void
+createBlackTuple(const struct pam * const pamP, 
+                 tuple *            const blackTupleP) {
+
+/* This is poorly named, because it lacks the "pnm" prefix.  But for some
+   reason, this is how we originally named this.  So to maintain backward
+   compatibility with binaries that refer to "createBlackTuple", we define
+   this.  The preferred name, pnm_createBlackTuple() was new in Netpbm 10.20,
+   January 2004.  We should eventually retire createBlackTuple().
+*/
+    pnm_createBlackTuple(pamP, blackTupleP);
+}
+
+
 tuple *
 pnm_allocpamrow(const struct pam * const pamP) {
 /*----------------------------------------------------------------------------
@@ -510,7 +525,7 @@ pnm_readpaminit(FILE *file, struct pam * const pamP, const int size) {
         break;
         
     default:
-    pm_error("bad magic number - not a PAM, PPM, PGM, or PBM file");
+        pm_error("bad magic number - not a PAM, PPM, PGM, or PBM file");
     }
     
     if (pamP->maxval >> 8 == 0)       pamP->bytes_per_sample = 1;
@@ -523,44 +538,15 @@ pnm_readpaminit(FILE *file, struct pam * const pamP, const int size) {
 
 
 
-static sample
-get_sample(FILE *file, const int bytes) {
-
-    sample value;  /* our return value */
-
-    if (bytes == 1) {
-        /* Here's a speedup for the common 1-byte sample case: */
-        value = getc(file);
-        if (value == EOF)
-            pm_error("EOF/error reading 1 byte sample from file.");
-    } else {
-        /* This code works for bytes == 1..4 */
-        /* We could speed this up by exploiting knowledge of the format of
-           an unsigned integer (i.e. endianness).  Then we could just cast
-           the value as an array of characters instead of shifting and
-           masking.
-           */
-        int shift;
-        unsigned char inval[4];
-        int cursor;
-        int n_read;
-
-        n_read = fread(inval, bytes, 1, file);
-        if (n_read < 1) 
-            pm_error("EOF/error reading %d byte sample from file.", bytes);
-        value = 0;  /* initial value */
-        cursor = 0;
-        for (shift = (bytes-1)*8; shift >= 0; shift-=8) 
-            value += inval[cursor++] << shift;
-    }
-    return(value);
-}
-
-
-
 void 
 pnm_readpamrow(const struct pam * const pamP, tuple* const tuplerow) {
 
+    /* For speed, we don't check any of the inputs for consistency 
+       here (unless it's necessary to avoid crashing).  Any consistency
+       checking should have been done by a prior call to 
+       pnm_writepaminit().
+    */
+
     /* Need a special case for raw PBM because it has multiple tuples (8)
        packed into one byte.
        */
@@ -585,7 +571,7 @@ pnm_readpamrow(const struct pam * const pamP, tuple* const tuplerow) {
                     tuplerow[col][samp] = pm_getuint(pamP->file);
                 else 
                     tuplerow[col][samp] = 
-                        get_sample(pamP->file, pamP->bytes_per_sample);
+                        pm_getraw(pamP->file, pamP->bytes_per_sample);
             }
         }
     }
@@ -611,19 +597,6 @@ pnm_readpam(FILE *file, struct pam * const pamP, const int size) {
 
 
 
-void 
-pnm_writepam(struct pam *const pamP, tuple ** const tuplearray) {
-
-    int row;
-
-    pnm_writepaminit(pamP);
-    
-    for (row = 0; row < pamP->height; row++) 
-        pnm_writepamrow(pamP, tuplearray[row]);
-}
-
-
-
 void 
 pnm_writepaminit(struct pam *const pamP) {
 
@@ -700,39 +673,6 @@ pnm_writepaminit(struct pam *const pamP) {
 
             
 
-static void
-write_sample(FILE *file, const unsigned int value, const int bytes) {
-
-    if (bytes == 1) {
-        /* Here's a speedup for the common 1-byte sample case: */
-        int rc;
-        rc = fputc(value, file);
-        if (rc == EOF)
-            pm_error("Error writing 1 byte sample to file.");
-    } else {
-        /* This code works for bytes == 1..4 */
-        /* We could speed this up by exploiting knowledge of the format of
-           an unsigned integer (i.e. endianness).  Then we could just cast
-           the value as an array of characters instead of shifting and
-           masking.
-           */
-        int shift;
-        unsigned char outval[4];
-        int cursor;
-        int n_written;
-
-        cursor = 0;
-        for (shift = (bytes-1)*8; shift >= 0; shift-=8) {
-            outval[cursor++] = (value >> shift) & 0xFF;
-        }
-        n_written = fwrite(&outval, bytes, 1, file);
-        if (n_written == 0) 
-            pm_error("Error writing %d byte sample to file.", bytes);
-    }
-}
-
-
-
 void 
 pnm_writepamrow(const struct pam *const pamP, const tuple * const tuplerow) {
 
@@ -740,7 +680,7 @@ pnm_writepamrow(const struct pam *const pamP, const tuple * const tuplerow) {
        here (unless it's necessary to avoid crashing).  Any consistency
        checking should have been done by a prior call to 
        pnm_writepaminit().
-       */
+    */
 
     /* Need a special case for raw PBM because it has multiple tuples (8)
        packed into one byte.
@@ -758,15 +698,15 @@ pnm_writepamrow(const struct pam *const pamP, const tuple * const tuplerow) {
         int col;
 
         for (col = 0; col < pamP->width; col++) {
-            int samp;
-            for (samp = 0; samp < pamP->depth; samp++) {
+            unsigned int samp;
+            for (samp = 0; samp < pamP->depth; ++samp) {
                 /* If we allowed writing of plain format via the pam functions,
                    we could do a putus() here to write a plain sample.  We'd
                    also need some stuff to put in the spaces and line breaks.
                    */
-                write_sample(pamP->file, 
-                             tuplerow[col][samp], 
-                             pamP->bytes_per_sample);
+                pm_putraw(pamP->file, 
+                          tuplerow[col][samp], 
+                          pamP->bytes_per_sample);
             }
         }    
     }
@@ -774,6 +714,19 @@ pnm_writepamrow(const struct pam *const pamP, const tuple * const tuplerow) {
 
 
 
+void 
+pnm_writepam(struct pam *const pamP, tuple ** const tuplearray) {
+
+    int row;
+
+    pnm_writepaminit(pamP);
+    
+    for (row = 0; row < pamP->height; row++) 
+        pnm_writepamrow(pamP, tuplearray[row]);
+}
+
+
+
 void
 pnm_checkpam(const struct pam *   const pamP, 
              enum pm_check_type   const check_type, 
@@ -808,59 +761,6 @@ pnm_checkpam(const struct pam *   const pamP,
 
 
 
-double pnm_lumin_factor[3] = {PPM_LUMINR, PPM_LUMING, PPM_LUMINB};
-
-void
-pnm_YCbCrtuple(const tuple tuple, 
-               double * const YP, double * const CbP, double * const CrP) {
-/*----------------------------------------------------------------------------
-   Assuming that the tuple 'tuple' is of tupletype RGB, return the 
-   Y/Cb/Cr representation of the color represented by the tuple.
------------------------------------------------------------------------------*/
-    const int red = tuple[PAM_RED_PLANE];
-    const int grn = tuple[PAM_GRN_PLANE];
-    const int blu = tuple[PAM_BLU_PLANE];
-    
-    *YP  = (+ PPM_LUMINR * red + PPM_LUMING * grn + PPM_LUMINB * blu);
-    *CbP = (- 0.16874 * red - 0.33126 * grn + 0.50000 * blu);
-    *CrP = (+ 0.50000 * red - 0.41869 * grn - 0.08131 * blu);
-}
-
-
-
-void 
-pnm_YCbCr_to_rgbtuple(const struct pam * const pamP,
-                      tuple              const tuple,
-                      double             const Y,
-                      double             const Cb, 
-                      double             const Cr,
-                      int *              const overflowP) {
-
-    double rgb[3];
-    unsigned int plane;
-    bool overflow;
-
-    rgb[PAM_RED_PLANE] = Y + 1.4022 * Cr + .5;
-    rgb[PAM_GRN_PLANE] = Y - 0.7145 * Cr - 0.3456 * Cb + .5;
-    rgb[PAM_BLU_PLANE] = Y + 1.7710 * Cb + .5;
-
-    overflow = FALSE;  /* initial assumption */
-
-    for (plane = 0; plane < 3; ++plane) {
-        if (rgb[plane] > pamP->maxval) {
-            overflow = TRUE;
-            tuple[plane] = pamP->maxval;
-        } else if (rgb[plane] < 0) {
-            overflow = TRUE;
-            tuple[plane] = 0;
-        } else 
-            tuple[plane] = rgb[plane];
-    }
-    if (overflowP)
-        *overflowP = overflow;
-}
-
-
 /*=============================================================================
    pm_system() Standard Input feeder and Standard Output accepter functions.   
 =============================================================================*/
