@@ -10,6 +10,12 @@
 ** implied warranty.
 */
 
+/* See libpbm.c for the complicated explanation of this 32/64 bit file
+   offset stuff.
+*/
+#define _FILE_OFFSET_BITS 64
+#define _LARGE_FILES  
+
 #include <string.h>
 #include <stdio.h>
 #include <errno.h>
@@ -32,9 +38,11 @@ pgm_nextimage(FILE * const file, int * const eofP) {
 }
 
 
+
 void
 pgm_readpgminitrest(FILE * const file, 
-                    int * const colsP, int * const rowsP, 
+                    int *  const colsP, 
+                    int *  const rowsP, 
                     gray * const maxvalP) {
 
     gray maxval;
@@ -46,8 +54,11 @@ pgm_readpgminitrest(FILE * const file,
     /* Read maxval. */
     maxval = pm_getuint(file);
     if (maxval > PGM_OVERALLMAXVAL)
-        pm_error( "maxval is too large.  The largest we can handle is %u.", 
-                  PGM_OVERALLMAXVAL );
+        pm_error("maxval of input image (%u) is too large.  "
+                 "The maximum allowed by PGM is %u.", 
+                 maxval, PGM_OVERALLMAXVAL);
+    if (maxval == 0)
+        pm_error("maxval of input image is zero.");
 
     *maxvalP = maxval;
 }
