@@ -651,6 +651,57 @@ pm_openw(const char * const name) {
 
 
 
+FILE * 
+pm_tmpfile(void) {
+
+    int fd;
+    FILE * retval;
+    char buf[FILENAME_MAX];
+    const char * tbuf;
+    unsigned int fnamelen;
+
+    fnamelen = strlen (pm_progname) + 10; /* "/" + "_XXXXXX\0" */
+
+    tbuf = getenv("TMPDIR");
+
+    if ((tbuf != NULL) && (strlen(tbuf) > FILENAME_MAX - fnamelen))
+        /* length of TMPDIR value too big for buf */
+        tbuf = NULL;
+
+    buf[FILENAME_MAX - fnamelen -1] = 0;
+
+    if ((tbuf == NULL) || (tbuf[0] == 0))
+        /* ENV string not suitable for evaluation of tmp-file */
+        strncpy(buf, P_tmpdir, FILENAME_MAX - fnamelen);
+    else
+        strncpy(buf, tbuf, FILENAME_MAX - fnamelen);
+
+    if (buf[strlen(buf) - 1] != '/')
+        strcat (buf, "/");
+    
+    strcat(buf, pm_progname);
+    strcat(buf, "_XXXXXX");
+
+    fd = mkstemp(buf);
+
+    if (fd < 0)
+        pm_error("Unable to create temporary file.  mkstemp() failed with "
+                 "errno %d (%s)", errno, strerror(errno));
+    else {
+        unlink(buf);
+
+        retval = fdopen(fd, "w+b");
+
+        if (retval == NULL)
+            pm_error("Unable to create temporary file.  fdopen() failed "
+                     "with errno %d (%s)", errno, strerror(errno));
+    }
+
+    return retval;
+}
+
+
+
 FILE *
 pm_openr_seekable(const char name[]) {
 /*----------------------------------------------------------------------------
@@ -693,8 +744,8 @@ pm_openr_seekable(const char name[]) {
     if (seekable) {
         seekable_file = original_file;
     } else {
-        seekable_file = tmpfile();
-
+        seekable_file = pm_tmpfile();
+        
         /* Copy the input into the temporary seekable file */
         while (!feof(original_file) && !ferror(original_file) 
                && !ferror(seekable_file)) {
