@@ -10,6 +10,12 @@
 ** implied warranty.
 */
 
+/* See libpbm.c for the complicated explanation of this 32/64 bit file
+   offset stuff.
+*/
+#define _FILE_OFFSET_BITS 64
+#define _LARGE_FILES  
+
 #include <string.h>
 #include <stdio.h>
 #include <errno.h>
@@ -35,25 +41,30 @@ ppm_nextimage(FILE *file, int * const eofP) {
 }
 
 
+
 void
 ppm_readppminitrest(FILE *   const file, 
                     int *    const colsP, 
                     int *    const rowsP, 
-                    pixval * const maxvalP ) {
-        unsigned int maxval;
+                    pixval * const maxvalP) {
+    unsigned int maxval;
 
     /* Read size. */
-    *colsP = (int)pm_getuint( file );
-    *rowsP = (int)pm_getuint( file );
+    *colsP = (int)pm_getuint(file);
+    *rowsP = (int)pm_getuint(file);
 
     /* Read maxval. */
-    maxval = pm_getuint( file );
-    if ( maxval > PPM_OVERALLMAXVAL )
-	pm_error("maxval (%d) is too large.\n"
-             "The maximum allowed by the PPM is %d.",
-             maxval, PPM_OVERALLMAXVAL); 
+    maxval = pm_getuint(file);
+    if (maxval > PPM_OVERALLMAXVAL)
+        pm_error("maxval of input image (%u) is too large.  "
+                 "The maximum allowed by the PPM is %u.",
+                 maxval, PPM_OVERALLMAXVAL); 
+    if (maxval == 0)
+        pm_error("maxval of input image is zero.");
+
     *maxvalP = maxval;
-    }
+}
+
 
 
 void
